from pii_anonymizer.standalone.analyze.detectors.base_detector import BaseDetector
from pii_anonymizer.standalone.analyze.utils.regex import RegEx


class NationalIdDetector(BaseDetector):
    def __init__(self):
        self.name = "NRIC"
        self.pattern = (
            RegEx()
            .one_of("STFG")
            .any_digit()
            .num_occurrences(7)
            .range("A", "Z")
            .build()
        )

    def get_name(self):
        return self.name

    def get_pattern(self):
        return self.pattern

    def __get_offset(self, text):
        return 4 if text in "TG" else 0

    def __is_NRIC(self, text, loc):
        if text[0] in "ST":
            return "JZIHGFEDCBA"[loc] == text[8]
        return False

    def __is_FIN(self, text, loc):
        if text[0] in "FG":
            return "XWUTRQPNMLK"[loc] == text[8]
        return False

    def validate(self, text):
        weight = self.__get_weight(text)
        first_character = text[0]
        offset = self.__get_offset(first_character)
        loc = (offset + weight) % 11
        return self.__is_NRIC(text, loc) or self.__is_FIN(text, loc)

    def __get_weight(self, text):
        numbers = [int(digit) for digit in list(text[1:-1])]
        for index, i in enumerate(numbers):
            if index == 0:
                numbers[index] *= 2
            numbers[index] *= 8 - index
        return sum(numbers)



class RegEx:
    def __init__(self):
        self.regex_string = ""

    def __is_numeric(self, value):
        return isinstance(value, int)

    def __is_single_character_value(self, value):
        return len(str(value)) == 1

    def __validate_range(self, start, end):
        if start > end:
            raise ValueError("Range start should be less than end")

    def boundary(self):
        self.regex_string += "\\b"
        return self

    def pipe(self):
        self.regex_string += "|"
        return self

    def range(self, from_char, to_char):
        if not self.__is_single_character_value(
            from_char
        ) or not self.__is_single_character_value(to_char):
            raise ValueError("Range boundaries should be single character")

        self.__validate_range(from_char, to_char)
        self.regex_string += "[{}-{}]".format(from_char, to_char)
        return self

    def one_of(self, character_set):
        if character_set is None or character_set == "":
            raise ValueError("Character Set should not be empty")

        self.regex_string += "[" + character_set + "]"
        return self

    def any_digit(self):
        self.regex_string += "\\d"
        return self

    def num_occurrences(self, number):
        if number < 1:
            raise ValueError

        self.regex_string += "{" + str(number) + "}"
        return self

    def one_or_more_occurrences(self):
        self.regex_string += "+"
        return self

    def zero_or_more_occurrences(self):
        self.regex_string += "*"
        return self

    def zero_or_one_occurrences(self):
        self.regex_string += "?"
        return self

    def range_occurrences(self, start, end):
        if not self.__is_numeric(start) or not self.__is_numeric(end):
            raise TypeError("Range should be integers")

        self.__validate_range(start, end)
        self.regex_string += "{" + str(start) + "," + str(end) + "}"
        return self

    def literal(self, literal):
        self.regex_string += literal
        return self

    def build(self):
        return self.regex_string



from unittest import TestCase
import os
import pandas as pd
from pii_anonymizer.standalone.acquire.csv_parser import CsvParser


class TestCsvParser(TestCase):
    def setUp(self):
        self.current_dir = os.path.dirname(os.path.realpath(__file__))

    def test_invalid_config_gets_caught_during_initialization(self):
        context = {}
        with self.assertRaises(ValueError) as ve:
            CsvParser(config=context)
        self.assertEqual(
            str(ve.exception), "Config 'file_path' needs to be provided for parsing"
        )

    def test_if_valid_csv_file_provided_returns_pandas_df(self):
        file_path = "{}/data/comma_delimited_file.csv".format(self.current_dir)
        config = {"file_path": file_path, "delimiter": ""}
        test_csv_parser_valid_file_path = CsvParser(config=config)
        expected = pd.DataFrame({"name": ["Lisa Beard"], "ssn": ["557-39-2479"]})
        actual = test_csv_parser_valid_file_path.parse()
        self.assertEqual(actual.to_dict(), expected.to_dict())

    def test_if_valid_csv_file_with_different_delimiter_provided_returns_pandas_df(
        self,
    ):
        file_path = "{}/data/pipe_delimited_file.csv".format(self.current_dir)
        config = {"file_path": file_path, "delimiter": "|"}
        test_csv_parser_valid_file_path = CsvParser(config=config)
        expected = pd.DataFrame({"name": ["Lisa Beard"], "ssn": ["557-39-2479"]})
        actual = test_csv_parser_valid_file_path.parse()
        self.assertEqual(actual.to_dict(), expected.to_dict())

    def test_if_empty_csv_file_returns_empty_pandas_df(self):
        file_path = "{}/data/empty.csv".format(self.current_dir)
        config = {"file_path": file_path}
        test_csv_parser_valid_file_path = CsvParser(config=config)
        expected = pd.DataFrame({})
        actual = test_csv_parser_valid_file_path.parse()
        self.assertEqual(actual.to_dict(), expected.to_dict())

    def test_if_error_is_raised_if_df_has_null_values(self):
        file_path = "{}/data/missing_comma.csv".format(self.current_dir)
        config = {"file_path": file_path}
        with self.assertRaises(ValueError) as ve:
            CsvParser(config=config).parse()
        self.assertEqual(str(ve.exception), "Dataframe contains NULL values")



import importlib
import pkgutil
import inspect
import sys
from pyspark.sql import DataFrame
from pyspark.sql.types import StructField, StructType, ArrayType, StringType, LongType
from pii_anonymizer.spark.analyze.detectors.base_detector import BaseDetector
import pii_anonymizer.spark.analyze.detectors


class PIIDetector:
    def __init__(self):
        self.detectors = self.__get_detector_instances()

    def __get_detector_modules(self):
        modules = [
            modname
            for importer, modname, ispkg in pkgutil.walk_packages(
                path=pii_anonymizer.spark.analyze.detectors.__path__,
                prefix=pii_anonymizer.spark.analyze.detectors.__name__ + ".",
            )
            if "tests" not in modname
        ]
        return modules

    def __get_detector_instances(self):
        modules = self.__get_detector_modules()
        detectors = []
        for module in modules:
            importlib.import_module(module)
            classes = inspect.getmembers(sys.modules[module], inspect.isclass)
            for class_name, class_type in classes:
                if class_name != "BaseDetector" and issubclass(
                    class_type, BaseDetector
                ):
                    detectors.append(class_type())
        return detectors

    def __detect_pii_row(self, row):
        new_row = []
        for element in row:
            results = []
            for detector in self.detectors:
                results += detector.execute(element)
            new_row.append(results)

        return new_row

    def get_analyzer_results(self, input_data_frame: DataFrame):
        columns = input_data_frame.columns

        array_structtype = StructType(
            [
                StructField("end", LongType(), False),
                StructField("start", LongType(), False),
                StructField("text", StringType(), False),
                StructField("type", StringType(), False),
            ]
        )
        result_schema = []
        for column in columns:
            result_schema.append(
                StructField(column, ArrayType(array_structtype, True), nullable=False)
            )

        result = input_data_frame.rdd.map(lambda x: self.__detect_pii_row(x)).toDF(
            schema=StructType(result_schema)
        )

        return result

    def _get_pii_list(self, row):
        get_analyzer_results_text = lambda x: x.text

        new_row = []
        for cell in row:
            pii_sublist = list(map(get_analyzer_results_text, cell))
            new_row.extend(pii_sublist)
        return new_row

    def get_redacted_text(self, input_data_frame: DataFrame, report: DataFrame):
        pii_list = report.rdd.flatMap(lambda row: self._get_pii_list(row)).collect()
        column = input_data_frame.columns
        result = input_data_frame.rdd.map(
            lambda row: self.__replace_redacted_text(row, pii_list)
        ).toDF(column)

        return result

    def __replace_redacted_text(self, row, pii_list):
        new_row = []
        for cell in row:
            for word in pii_list:
                if word in cell:
                    cell = cell.replace(word, "")
            new_row.append(cell)
        return new_row

    def analyze_data_frame(self, input_data_frame: DataFrame):
        report = self.get_analyzer_results(input_data_frame)
        redacted = self.get_redacted_text(input_data_frame, report)

        return report, redacted



import re
from abc import ABC, abstractmethod

from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult


class BaseDetector(ABC):
    def __init__(self):
        self.name = None
        self.pattern = None

    @abstractmethod
    def get_pattern(self):
        pass

    @abstractmethod
    def get_name(self):
        pass

    def validate(self, text):
        return True

    def execute(self, text):
        results = []
        matches = re.finditer(self.get_pattern(), text)
        for match in matches:
            matched_string = match.string[match.start() : match.end()]
            if self.validate(matched_string):
                results.append(
                    AnalyzerResult(
                        matched_string, self.get_name(), match.start(), match.end()
                    )
                )
        return results



from pii_anonymizer.standalone.write.csv_writer import CsvWriter
from unittest import TestCase

class TestCsvWriter(TestCase):

    # TODO: check acquire file path exists
    def test_invalid_config_gets_caught_during_initialization(self):
        context = {}
        with self.assertRaises(ValueError) as ve:
            CsvWriter(config=context)
        self.assertEqual(
            str(ve.exception),
            "Config 'output_file_path' needs to be provided for parsing",
        )

    def test_correct_output_path_is_generated(self):
        context = {
            "acquire": {"file_path": "/anonymizer/test_data.csv", "delimiter": ","},
            "anonymize": {"output_file_path": "/anonymizer/output"},
        }
        input_file_name = "test_data"
        output_directory = "/anonymizer/output"
        expected = f"{output_directory}/{input_file_name}_anonymized.csv"
        writer = CsvWriter(config=context)
        self.assertEqual(writer.get_output_file_path(), expected)



class AnalyzerResult:
    def __init__(self, text, type, start, end):
        self.text = text
        self.type = type
        self.start = start
        self.end = end

    def __eq__(self, other):
        return (
            type(self) == type(other)
            and self.text == other.text
            and self.type == other.type
            and self.start == other.start
            and self.end == other.end
        )

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return "Text {} at position ({},{}) was identified as {}".format(
            self.text, self.start, self.end, self.type
        )

    def detector(self):
        return self.type



import json
import os
from unittest import TestCase
from unittest.mock import patch, MagicMock

import pandas as pd

from pii_anonymizer.standalone.constants import ACQUIRE, REPORT
from pii_anonymizer.standalone.dpf_main import DPFMain


class TestDPFMain(TestCase):
    def setUp(self):
        test_config = "{}/{}".format(
            os.path.dirname(os.path.realpath(__file__)), "config/test_config.json"
        )
        self.dpf_main = DPFMain(test_config)
        with open(test_config) as input_file:
            self.config_json = json.load(input_file)

    @patch("pii_anonymizer.standalone.write.csv_writer.CsvWriter.write_csv")
    @patch("pii_anonymizer.standalone.write.csv_writer.CsvWriter.__init__")
    @patch("pii_anonymizer.standalone.report.report_generator.ReportGenerator.generate")
    @patch("pii_anonymizer.standalone.report.report_generator.ReportGenerator.__init__")
    @patch(
        "pii_anonymizer.standalone.analyze.detectors.pii_detector.PIIDetector.analyze_data_frame"
    )
    @patch("pii_anonymizer.standalone.acquire.csv_parser.CsvParser.parse")
    @patch("pii_anonymizer.standalone.acquire.csv_parser.CsvParser.__init__")
    def test_run_parses_the_config_file_and_invokes_respective_stages_correctly(
        self,
        mock_csv_parser_init,
        mock_csv_parser_parse,
        mock_pii_analyze_df,
        mock_report_generator_init,
        mock_generate_report,
        mock_csv_writer_init,
        mock_csv_writer_write_csv,
    ):
        mock_csv_parser_init.return_value = None
        mock_csv_parser_parse.return_value = MagicMock()
        mock_pii_analyze_df.return_value = (
            pd.DataFrame({"summary": ["test result"]}),
            pd.DataFrame({}),
        )
        mock_report_generator_init.return_value = None
        mock_generate_report.return_value = MagicMock()
        mock_csv_writer_init.return_value = None
        mock_csv_writer_write_csv.return_value = None
        self.dpf_main.run()
        mock_csv_parser_init.assert_called_with(config=self.config_json[ACQUIRE])
        mock_csv_parser_parse.assert_called_with()
        mock_pii_analyze_df.assert_called_with(mock_csv_parser_parse.return_value)
        mock_report_generator_init.assert_called_with(config=self.config_json[REPORT])
        mock_generate_report.assert_called_with(
            results_df=mock_pii_analyze_df.return_value[0]
        )
        mock_csv_writer_init.assert_called_with(config=self.config_json)
        mock_csv_writer_write_csv.assert_called_with(
            df=mock_pii_analyze_df.return_value[1]
        )

    @patch("pii_anonymizer.standalone.write.csv_writer.CsvWriter.write_csv")
    @patch("pii_anonymizer.standalone.write.csv_writer.CsvWriter.__init__")
    @patch("pii_anonymizer.standalone.report.report_generator.ReportGenerator.generate")
    @patch(
        "pii_anonymizer.standalone.analyze.detectors.pii_detector.PIIDetector.analyze_data_frame"
    )
    @patch("pii_anonymizer.standalone.acquire.csv_parser.CsvParser.parse")
    @patch("pii_anonymizer.standalone.acquire.csv_parser.CsvParser.__init__")
    def test_run_short_circuits_generate_report_when_no_PII_values_detected(
        self,
        mock_csv_parser_init,
        mock_csv_parser_parse,
        mock_pii_analyze_df,
        mock_generate_report,
        mock_csv_writer_init,
        mock_csv_writer_write_csv,
    ):
        mock_csv_parser_init.return_value = None
        mock_csv_parser_parse.return_value = pd.DataFrame({})
        mock_pii_analyze_df.return_value = (pd.DataFrame({}), pd.DataFrame({}))
        mock_generate_report.return_value = MagicMock()
        mock_generate_report.return_value = None
        mock_csv_writer_init.return_value = None
        mock_csv_writer_write_csv.return_value = None
        self.dpf_main.run()
        mock_csv_parser_init.assert_called_with(config=self.config_json[ACQUIRE])
        mock_csv_parser_parse.assert_called_with()
        mock_pii_analyze_df.assert_called_with(mock_csv_parser_parse.return_value)
        mock_generate_report.assert_not_called()
        mock_csv_writer_init.assert_called_with(config=self.config_json)
        mock_csv_writer_write_csv.assert_called_with(
            df=mock_pii_analyze_df.return_value[1]
        )



from pii_anonymizer.standalone.dpf_main import DPFMain
from pii_anonymizer.common.get_args import get_args


def main():
    config_file_path = get_args()
    DPFMain(config_file_path).run()

if __name__ == "__main__":
    main()


from unittest import TestCase

from pii_anonymizer.standalone.analyze.detectors.phone_number_detector import (
    PhoneNumberDetector,
)
from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult


class TestPhoneNumberDetector(TestCase):
    def setUp(self):
        self.phone_number_detector = PhoneNumberDetector()

    def test_default_property_values_are_correct(self):
        self.assertEqual("PHONE_NUMBER", self.phone_number_detector.name)
        self.assertEqual(
            "(\\+65?\\s?[689]\\d{7})|"
            "(\\+65?\\s?[689]\\d{3} \\d{4})|"
            "([689]\\d{7})|"
            "([689]\\d{3} \\d{4})|"
            "([(]65[)]\\s?[689]\\d{7})|"
            "([(]65[)]\\s?[689]\\d{3} \\d{4})",
            self.phone_number_detector.pattern,
        )

    def test_invalid_phone_number_does_not_get_detected(self):
        self.assertEqual(len(self.phone_number_detector.execute("S0000001I")), 0)

    def __assert_single_result(self, text_to_be_tested, start, end):
        actual = self.phone_number_detector.execute(text_to_be_tested)
        expected = AnalyzerResult(text_to_be_tested, "PHONE_NUMBER", start, end)
        self.assertEqual(len(actual), 1)
        self.assertEqual(expected, actual[0])

    def test_valid_phone_number_gets_detected_correctly(self):
        self.__assert_single_result("+65 65781234", 0, 12)
        self.__assert_single_result("+65 85781234", 0, 12)
        self.__assert_single_result("+65 95781234", 0, 12)

        self.__assert_single_result("+65 6578 1234", 0, 13)
        self.__assert_single_result("+65 8578 1234", 0, 13)
        self.__assert_single_result("+65 9578 1234", 0, 13)

        self.__assert_single_result("65781234", 0, 8)
        self.__assert_single_result("85781234", 0, 8)
        self.__assert_single_result("95781234", 0, 8)

        self.__assert_single_result("6578 1234", 0, 9)
        self.__assert_single_result("8578 1234", 0, 9)
        self.__assert_single_result("9578 1234", 0, 9)

        self.__assert_single_result("(65) 65781234", 0, 13)
        self.__assert_single_result("(65) 85781234", 0, 13)
        self.__assert_single_result("(65) 95781234", 0, 13)

        self.__assert_single_result("(65) 6578 1234", 0, 14)
        self.__assert_single_result("(65) 8578 1234", 0, 14)
        self.__assert_single_result("(65) 9578 1234", 0, 14)



class AnonymizerResult:
    def __init__(self, redacted_text, analyzer_results):
        self.redacted_text = redacted_text
        self.analyzer_results = analyzer_results

    def __eq__(self, other):
        return (
            type(self) == type(other)
            and self.redacted_text == other.redacted_text
            and self.analyzer_results == other.analyzer_results
        )

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return "PII information found: \n{}\nRedacted text: {}".format(
            self.analyzer_results, self.redacted_text
        )



import os
from unittest import TestCase
from pyspark.sql import SparkSession
from pyspark.sql.types import StructType
from pii_anonymizer.spark.acquire.csv_parser import CsvParser


class TestCsvParser(TestCase):
    def setUp(self) -> None:
        self.SPARK = (
            SparkSession.builder.master("local").appName("Test CSVParser").getOrCreate()
        )
        self.current_dir = os.path.dirname(os.path.realpath(__file__))

    def test_invalid_config_gets_caught_during_initialization(self):
        context = {}
        with self.assertRaises(ValueError) as ve:
            CsvParser(self.SPARK, config=context)
        self.assertEqual(
            str(ve.exception), "Config 'file_path' needs to be provided for parsing"
        )

    def test_if_valid_csv_file_provided_returns_spark_df(self):
        file_path = "{}/data/comma_delimited_file.csv".format(self.current_dir)
        config = {"file_path": file_path, "delimiter": ""}

        expected = self.SPARK.createDataFrame(
            [("Lisa Beard", "557-39-2479")], ["name", "ssn"]
        )
        actual = CsvParser(spark=self.SPARK, config=config).parse()

        self.assertEqual(actual.schema, expected.schema)
        self.assertEqual(actual.collect(), expected.collect())

    def test_if_valid_csv_file_with_different_delimiter_provided_returns_spark_df(self):
        file_path = "{}/data/pipe_delimited_file.csv".format(self.current_dir)
        config = {"file_path": file_path, "delimiter": "|"}

        expected = self.SPARK.createDataFrame(
            [("Lisa Beard", "557-39-2479")], ["name", "ssn"]
        )
        actual = CsvParser(spark=self.SPARK, config=config).parse()

        self.assertEqual(actual.schema, expected.schema)
        self.assertEqual(actual.collect(), expected.collect())

    def test_if_empty_csv_file_returns_empty_pandas_df(self):
        file_path = "{}/data/empty.csv".format(self.current_dir)
        config = {"file_path": file_path}
        expected = self.SPARK.createDataFrame([], StructType([]))
        actual = CsvParser(spark=self.SPARK, config=config).parse()
        self.assertEqual(actual.schema, expected.schema)
        self.assertEqual(actual.collect(), expected.collect())



from unittest import TestCase

from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult


class TestAnalyzerResult(TestCase):
    def test_equality(self):
        expected = AnalyzerResult("text", "type", 0, 10)
        actual = AnalyzerResult("text", "type", 0, 10)
        self.assertEqual(expected, actual)

    def test_inequality(self):
        self.assertNotEqual(
            AnalyzerResult("text", "type", 0, 10),
            AnalyzerResult("different_text", "type", 0, 10),
        )
        self.assertNotEqual(
            AnalyzerResult("text", "type", 0, 10),
            AnalyzerResult("text", "different_type", 0, 10),
        )
        self.assertNotEqual(
            AnalyzerResult("text", "type", 0, 10), AnalyzerResult("text", "type", 1, 10)
        )
        self.assertNotEqual(
            AnalyzerResult("text", "type", 0, 10), AnalyzerResult("text", "type", 0, 11)
        )

    def test_repr(self):
        expected = "Text sample_data at position (0,10) was identified as type"
        self.assertEqual(
            AnalyzerResult("sample_data", "type", 0, 10).__repr__(), expected
        )

    def test_str(self):
        expected = "Text sample_data at position (0,10) was identified as type"
        self.assertEqual(str(AnalyzerResult("sample_data", "type", 0, 10)), expected)

    def test_get_detector_fetches_detector_type_correctly(self):
        result = AnalyzerResult("text", "EMAIL", 0, 10)
        self.assertEqual(result.detector(), "EMAIL")



from pii_anonymizer.spark.analyze.detectors.base_detector import BaseDetector
from pii_anonymizer.spark.analyze.utils.regex import RegEx


class CreditCardDetector(BaseDetector):
    def __init__(self):
        self.name = "CREDIT_CARD"
        self.pattern = (
            RegEx()
            .literal("4")
            .any_digit()
            .num_occurrences(3)
            .pipe()
            .literal("5")
            .range(0, 5)
            .any_digit()
            .num_occurrences(2)
            .pipe()
            .literal("6")
            .any_digit()
            .num_occurrences(3)
            .pipe()
            .literal("1")
            .any_digit()
            .num_occurrences(3)
            .pipe()
            .literal("3")
            .any_digit()
            .num_occurrences(3)
            .one_of("- ")
            .zero_or_one_occurrences()
            .any_digit()
            .range_occurrences(3, 4)
            .one_of("- ")
            .zero_or_one_occurrences()
            .any_digit()
            .range_occurrences(3, 4)
            .one_of("- ")
            .zero_or_one_occurrences()
            .any_digit()
            .range_occurrences(3, 5)
            .build()
        )

    def get_name(self):
        return self.name

    def get_pattern(self):
        return self.pattern

    def validate(self, text):
        def digits_of(n):
            return [int(d) for d in str(n)]

        digits = digits_of(text.replace("-", "").replace(" ", ""))
        odd_digits = digits[-1::-2]
        even_digits = digits[-2::-2]
        checksum = sum(odd_digits)

        for d in even_digits:
            checksum += sum(digits_of(d * 2))

        return checksum % 10 == 0



from unittest import TestCase

from pii_anonymizer.standalone.analyze.detectors.national_id_detector import (
    NationalIdDetector,
)


class TestNationalIdDetector(TestCase):
    def setUp(self):
        self.national_id_detector = NationalIdDetector()

    def test_default_property_values_are_correct(self):
        self.assertEqual("NRIC", self.national_id_detector.name)
        self.assertEqual("[STFG]\\d{7}[A-Z]", self.national_id_detector.pattern)

    def test_execute_return_true_when_valid_old_NRIC(self):
        self.assertTrue(self.national_id_detector.validate("S0000001I"))

    def test_execute_return_true_when_valid_old_FIN(self):
        self.assertTrue(self.national_id_detector.validate("F0000001U"))

    def test_execute_return_true_when_valid_new_NRIC(self):
        self.assertTrue(self.national_id_detector.validate("T0000001E"))

    def test_execute_return_true_when_valid_new_FIN(self):
        self.assertTrue(self.national_id_detector.validate("G0000001P"))

    def test_execute_return_false_when_invalid_old_NRIC(self):
        self.assertFalse(self.national_id_detector.validate("S0000001K"))

    def test_execute_return_false_when_invalid_new_NRIC(self):
        self.assertFalse(self.national_id_detector.validate("F0000001V"))

    def test_execute_return_false_when_invalid_old_FIN(self):
        self.assertFalse(self.national_id_detector.validate("T0000001F"))

    def test_execute_return_false_when_invalid_new_FIN(self):
        self.assertFalse(self.national_id_detector.validate("G0000001Q"))



from pyspark.sql import SparkSession, DataFrame
from pii_anonymizer.standalone.constants import OUTPUT_FILE_PATH, FILE_PATH


class CsvWriter:
    def __init__(self, spark: SparkSession, config):
        self.__validate_config(config)
        self.output_path = config["anonymize"][OUTPUT_FILE_PATH]
        self.input_file_name = config["acquire"][FILE_PATH]
        self.spark = spark

    def __validate_config(self, config):
        if (
            "anonymize" not in config
            or not config["anonymize"]
            or OUTPUT_FILE_PATH not in config["anonymize"]
            or not config["anonymize"][OUTPUT_FILE_PATH]
        ):
            raise ValueError(
                "Config 'output_file_path' needs to be provided for parsing"
            )

    def get_output_file_path(self):
        file_name = self.input_file_name.split("/")[-1]
        file_name_no_extension = file_name.split(".")[0]
        result = f"{self.output_path}/{file_name_no_extension}_anonymized"
        return result

    def write_csv(self, df: DataFrame):
        df.write.option("header","true").csv(self.get_output_file_path())



from unittest import TestCase

from pii_anonymizer.standalone.analyze.detectors.credit_card_detector import (
    CreditCardDetector,
)


class TestCreditCardDetector(TestCase):
    def setUp(self):
        self.credit_card_detector = CreditCardDetector()

    def test_default_property_values_are_correct(self):
        self.assertEqual("CREDIT_CARD", self.credit_card_detector.name)
        self.assertEqual(
            "4\\d{3}|5[0-5]\\d{2}|6\\d{3}|1\\d{3}|3\\d{3}[- ]?\\d{3,4}[- ]?\\d{3,4}[- ]?\\d{3,5}",
            self.credit_card_detector.pattern,
        )

    def test_valid_credit_cards(self):
        self.assertTrue(self.credit_card_detector.validate("4012888888881881"))
        self.assertTrue(self.credit_card_detector.validate("4012-8888-8888-1881"))
        self.assertTrue(self.credit_card_detector.validate("4012 8888 8888 1881"))

    def test_valid_airplus_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("122000000000003"))

    def test_valid_amex_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("371449635398431"))

    def test_valid_cartebleue_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("5555555555554444"))

    def test_valid_dankort_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("5019717010103742"))

    def test_valid_diners_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("30569309025904"))

    def test_valid_discover_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("6011000400000000"))

    def test_valid_jcb_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("3528000700000000"))

    def test_valid_maestro_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("6759649826438453"))

    def test_valid_mastercard_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("5555555555554444"))

    def test_valid_visa_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("4111111111111111"))

    def test_valid_visa_debit_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("4111111111111111"))

    def test_valid_visa_electron_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("4917300800000000"))

    def test_valid_visa_purchasing_credit_card(self):
        self.assertTrue(self.credit_card_detector.validate("4484070000000000"))

    def test_invalid_credit_card(self):
        self.assertFalse(self.credit_card_detector.validate("4012-8888-8888-1882"))

    def test_invalid_diners_card(self):
        self.assertFalse(self.credit_card_detector.validate("36168002586008"))



import os
import sys

sys.path.append(os.path.abspath("."))

import json

from pyspark.sql import SparkSession
from pii_anonymizer.spark.report.report_generator import ReportGenerator
from pii_anonymizer.spark.acquire.csv_parser import CsvParser
from pii_anonymizer.spark.analyze.detectors.pii_detector import PIIDetector
from pii_anonymizer.spark.constants import ACQUIRE, REPORT
from pii_anonymizer.spark.write.csv_writer import CsvWriter
from pii_anonymizer.common.get_args import get_args

class Main:
    def __init__(self, config_file_path):
        with open(config_file_path) as config_file:
            self.config = json.load(config_file)

    # TODO : validate the config for the stages right here
    def run(self):
        spark = (
            SparkSession.builder.master("local").appName("PIIDetector").getOrCreate()
        )
        parsed_data_frame = CsvParser(spark, config=self.config[ACQUIRE]).parse()
        pii_analysis_report, redacted_data_frame = PIIDetector().analyze_data_frame(
            parsed_data_frame
        )

        report_generator = ReportGenerator(config=self.config[REPORT])
        if report_generator.is_empty_report_dataframe(pii_analysis_report):
            print("NO PII VALUES WERE FOUND!")
        else:
            report_generator.generate(results_df=pii_analysis_report)
        CsvWriter(spark, config=self.config).write_csv(df=redacted_data_frame)

if __name__ == "__main__":
    args = get_args()
    Main(args).run()



from datetime import datetime
from enum import Enum
import os
import logging

import pandas as pd
from pyspark.sql.dataframe import DataFrame
from pyspark.sql.types import Row
from pii_anonymizer.spark.constants import LOCATION, REPORT_LEVEL


class ReportLevel(Enum):

    HIGH = "high"
    MEDIUM = "medium"
    LOW = "low"


class ReportGenerator:
    def __init__(self, config):
        self.report_file_location = config[LOCATION]
        self.report_level = config[REPORT_LEVEL]
        self.setup_logging_config()
        self.dataframe_is_empty = None

    def setup_logging_config(self):
        date = datetime.today().strftime("%Y%m%d")
        file_name = "{}/report_{}.log".format(self.report_file_location, date)
        if os.path.exists(file_name):
            mode = "a"
        else:
            if not os.path.exists(self.report_file_location):
                os.makedirs(self.report_file_location)
            mode = "x"
        file_handler = logging.FileHandler(filename=file_name, mode=mode)
        formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
        file_handler.setFormatter(formatter)
        logging.getLogger().addHandler(file_handler)
        logging.getLogger().setLevel(logging.INFO)

    def __generate_high_level_report(self, results_df: DataFrame):
        columns = results_df.columns
        report_df = pd.DataFrame({"Columns with PII values": columns})
        return report_df

    def __calculate_percentage(self, item_count, total_count):
        return round((item_count / total_count) * 100.0, 2)

    def _get_detector_results(self, row: Row, columns: list):
        new_row = []
        for index, cell in enumerate(row):
            current_col = columns[index]
            if cell != []:
                for analyzer_result in cell:
                    detector = analyzer_result["type"]
                    new_row.append(((current_col, detector), 1))
            else:
                new_row.append(((current_col, "no_pii"), 1))
        return new_row

    def __get_list_of_detectors(self, detector_results):
        report_detectors = []
        for key, _ in detector_results:
            detector = key[1]
            if detector not in report_detectors and detector != "no_pii":
                report_detectors.append(detector)
        return report_detectors

    def spark_generate_medium_level_report(self, results_df: DataFrame) -> pd.DataFrame:
        columns = results_df.columns
        detector_results = (
            results_df.rdd.flatMap(lambda row: self._get_detector_results(row, columns))
            .reduceByKey(lambda acc, next: acc + next)
            .collect()
        )
        report_detectors = self.__get_list_of_detectors(detector_results)
        num_rows = results_df.count()
        pd_columns = []
        for column in columns:
            detection_stats = self.__get_detection_stats(
                column, report_detectors, detector_results, num_rows
            )
            pd_columns.append(
                pd.Series(data=detection_stats, index=report_detectors, name=column)
            )
        report_df = pd.concat(pd_columns, axis=1).fillna(0)
        return report_df

    def __get_detection_stats(
        self,
        column: list,
        report_detectors: list,
        detector_results: list,
        num_rows: int,
    ) -> dict:
        detection_stats = {}
        default_value = ()
        for detector in report_detectors:
            column_detector_count = next(
                filter(
                    lambda result: result[0] == (column, detector), detector_results
                ),
                default_value,
            )
            if len(column_detector_count) > 0:
                count = column_detector_count[1]
                percentage_value = self.__calculate_percentage(
                    item_count=count, total_count=num_rows
                )
                detection_stats[detector] = (count, f"{percentage_value}%")
        return detection_stats

    def generate_report_content(self, results_df: DataFrame) -> pd.DataFrame:
        if self.report_level == ReportLevel.HIGH.value:
            return self.__generate_high_level_report(results_df)
        elif self.report_level == ReportLevel.MEDIUM.value:
            return self.spark_generate_medium_level_report(results_df)
        return self.spark_generate_medium_level_report(results_df)

    def __print(self, msg):
        formatted_msg = f"\n{msg}"
        print(formatted_msg)
        logging.info(formatted_msg)

    def __print_report(self, report):
        self.__print(
            "\n\n****************************PII ANALYSIS REPORT**************************\n\n"
        )
        if report.empty:
            self.__print("NO PII VALUES WERE FOUND!")
        else:
            self.__print(report)
        self.__print(
            "\n\n****************************DONE!**************************\n\n"
        )

    def generate(self, results_df: DataFrame):
        if self.is_empty_report_dataframe(results_df):
            print("NO PII VALUES WERE FOUND!")

        final_report = self.generate_report_content(results_df)
        self.__print_report(final_report)
        return final_report

    def is_empty_report_dataframe(self, results_df: DataFrame) -> bool:
        if self.dataframe_is_empty == None:
            self.dataframe_is_empty = results_df.rdd.flatMap(
                lambda row: self._row_is_empty_list(row)
            ).reduce(lambda acc, item: acc and item)
        return self.dataframe_is_empty

    def _row_is_empty_list(self, row: Row) -> map:
        return map(lambda cell: True if cell == [] else False, row)



import pandas as pd

from pii_anonymizer.standalone.constants import FILE_PATH


class CsvParser:
    def __init__(self, config):
        self.__validate_config(config)
        self.input_path = config["file_path"]
        self.delimiter = (
            config["delimiter"]
            if "delimiter" in config and config["delimiter"]
            else ","
        )

    def __validate_config(self, config):
        if FILE_PATH not in config or not config[FILE_PATH]:
            raise ValueError("Config 'file_path' needs to be provided for parsing")

    def parse(self):
        try:
            df = pd.read_csv(self.input_path, delimiter=self.delimiter)
        except pd.errors.EmptyDataError:
            return pd.DataFrame({})

        if df.isnull().values.any():
            raise ValueError("Dataframe contains NULL values")

        return df



from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult


class DropAnonymizer:
    @staticmethod
    def redact(text: str, analyzer_results: [AnalyzerResult]):
        for result in analyzer_results:
            text = text.replace(result.text, "")
        return text



from datetime import datetime
from enum import Enum

import os
import pandas as pd
import logging
from pii_anonymizer.standalone.constants import LOCATION, REPORT_LEVEL


class ReportLevel(Enum):

    HIGH = "high"
    MEDIUM = "medium"
    LOW = "low"


class ReportGenerator:
    def __init__(self, config):
        self.report_file_location = config[LOCATION]
        self.report_level = config[REPORT_LEVEL]
        self.setup_logging_config()

    def setup_logging_config(self):
        date = datetime.today().strftime("%Y%m%d")
        file_name = "{}/report_{}.log".format(self.report_file_location, date)
        if os.path.exists(file_name):
            mode = "a"
        else:
            if not os.path.exists(self.report_file_location):
                os.makedirs(self.report_file_location)
            mode = "x"
        file_handler = logging.FileHandler(filename=file_name, mode=mode)
        formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
        file_handler.setFormatter(formatter)
        logging.getLogger().addHandler(file_handler)
        logging.getLogger().setLevel(logging.INFO)

    def __generate_high_level_report(self, results_df):
        report_df = pd.DataFrame({"Columns with PII values": results_df.columns.values})
        return report_df

    def __collate_all_detectors_per_cell(self, analyzer_result):
        return [result.detector() for result in analyzer_result[1]]

    def __calculate_percentage(self, item_count, total_count):
        return round((item_count / total_count) * 100.0, 2)

    def __calculate_detector_percentage(self, row_count, count_map):
        percentage_map = {}
        for key, value in count_map.items():
            percentage_map[key] = "{}%".format(
                self.__calculate_percentage(value, row_count)
            )
        return percentage_map

    def __calculate_detector_count(self, column_series):
        detector_count_map = {}
        for analyzer_results in column_series.items():
            if not analyzer_results:
                continue
            detector_types = self.__collate_all_detectors_per_cell(analyzer_results)
            for detector_type in detector_types:
                if detector_type not in detector_count_map:
                    detector_count_map[detector_type] = 0
                detector_count_map[detector_type] += 1
        return detector_count_map

    # TODO : filter out the NAs before passing through this
    def calculate_detector_stats_for_each_column(self, column_series):
        stats_map = {}
        count_map = self.__calculate_detector_count(column_series)
        percentage_map = self.__calculate_detector_percentage(
            len(column_series), count_map
        )
        for key, value in count_map.items():
            stats_tuple = (value, percentage_map[key])
            stats_map[key] = stats_tuple
        return stats_map

    def __generate_medium_level_report(self, results_df):
        report_df = pd.DataFrame({})
        columns = list(results_df)
        column_reports = []
        for column in columns:
            detector_stats_for_each_column = (
                self.calculate_detector_stats_for_each_column(results_df[column])
            )
            column_report = pd.Series(
                detector_stats_for_each_column,
                name=column,
                index=detector_stats_for_each_column.keys(),
                dtype="object",
            )
            if not column_report.empty:
                column_reports.append(column_report)
        if column_reports:
            report_df = pd.concat(
                column_reports,
                axis=1,
                keys=[series.name for series in column_reports],
                sort=True,
            )
        return report_df.fillna(value=0)

    def generate_report_content(self, results_df):
        if self.report_level == ReportLevel.HIGH.value:
            return self.__generate_high_level_report(results_df)
        elif self.report_level == ReportLevel.MEDIUM.value:
            return self.__generate_medium_level_report(results_df)

    def __print(self, msg):
        print(msg)
        logging.info(msg)

    def __print_report(self, report):
        self.__print(
            "\n\n****************************PII ANALYSIS REPORT**************************\n\n"
        )
        if report.empty:
            self.__print("NO PII VALUES WERE FOUND!")
        else:
            self.__print(report)
        self.__print(
            "\n\n****************************DONE!**************************\n\n"
        )

    def generate(self, results_df):
        final_report = self.generate_report_content(results_df)
        self.__print_report(final_report)
        return final_report



from unittest import TestCase


from pii_anonymizer.standalone.analyze.detectors.base_detector import BaseDetector
from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult
from pii_anonymizer.standalone.analyze.utils.regex import RegEx


class TestBaseDetector(TestCase):
    def setUp(self):
        class TestClass(BaseDetector):
            def get_pattern(self):
                return (
                    RegEx()
                    .one_of("STFG")
                    .any_digit()
                    .num_occurrences(7)
                    .range("A", "Z")
                    .build()
                )

            def get_name(self):
                return "NRIC"

        self.test_class = TestClass()

    def test_execute_calls_match_and_validate(self):
        results = self.test_class.execute(
            "First President of Singapore NRIC was S0000001I"
        )
        self.assertEqual(len(results), 1)
        self.assertEqual(AnalyzerResult("S0000001I", "NRIC", 38, 47), results[0])

    def test_execute_returns_all_matches_when_more_than_one(self):
        results = self.test_class.execute(
            "First President of Singapore NRIC was S0000001I and the second president's was T0000001R"
        )
        self.assertEqual(len(results), 2)
        self.assertCountEqual(
            [
                AnalyzerResult("S0000001I", "NRIC", 38, 47),
                AnalyzerResult("T0000001R", "NRIC", 79, 88),
            ],
            results,
        )

    def test_execute_returns_empty_list_when_no_matches(self):
        results = self.test_class.execute(
            "First President of Singapore NRIC was ABC and the second president's was DEF"
        )
        self.assertEqual(len(results), 0)

    def test_get_name_and_get_patterns_are_abstract(self):
        with self.assertRaises(TypeError) as te:
            BaseDetector()
        self.assertEqual(
            str(te.exception),
            "Can't instantiate abstract class BaseDetector with abstract methods get_name, get_pattern",
        )



from unittest import TestCase
from pyspark.sql import SparkSession
from pii_anonymizer.spark.analyze.detectors.pii_detector import PIIDetector
from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult
from pyspark.sql.types import (
    StructField,
    StructType,
    ArrayType,
    StringType,
    LongType,
    Row,
)


class TestPIIDetector(TestCase):
    def setUp(self) -> None:
        self.SPARK = (
            SparkSession.builder.master("local")
            .appName("Test PIIDetector")
            .getOrCreate()
        )
        self.pii_detector = PIIDetector()

        self.array_structtype = StructType(
            [
                StructField("end", LongType(), False),
                StructField("start", LongType(), False),
                StructField("text", StringType(), False),
                StructField("type", StringType(), False),
            ]
        )
        self.schema = StructType(
            [
                StructField(
                    "summary", ArrayType(self.array_structtype, True), nullable=False
                ),
                StructField(
                    "phone number",
                    ArrayType(self.array_structtype, True),
                    nullable=False,
                ),
            ]
        )

    def test_analyze_data_frame_runs_analyze_against_each_cell_with_a_PII_value(self):
        test_data_frame = self.SPARK.createDataFrame(
            [
                (
                    "First President of Singapore NRIC was S0000001I",
                    "Some examples of phone numbers are +65 62345678",
                ),
                (
                    "A typical email id would look something like test@sample.com",
                    "Some examples of phone numbers are +65 62345678",
                ),
            ],
            ["summary", "phone number"],
        )

        actual = self.pii_detector.get_analyzer_results(test_data_frame)

        expected_data_frame = self.SPARK.createDataFrame(
            [
                (
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ),
                (
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ),
            ],
            self.schema,
        )

        self.assertEqual(actual.schema, expected_data_frame.schema)
        self.assertEqual(actual.collect(), expected_data_frame.collect())

    def test_analyze_data_frame_runs_analyze_against_cell_with_multiple_PII_values(
        self,
    ):
        test_data_frame = self.SPARK.createDataFrame(
            [
                (
                    "First President of Singapore NRIC was S0000001I",
                    "Some examples of phone numbers are +65 62345678",
                ),
                (
                    "email test@sample.com and phone +65 62345678",
                    "Phone one +65 62345678 Phone two +65 62345678",
                ),
            ],
            ["summary", "phone number"],
        )

        actual = self.pii_detector.get_analyzer_results(test_data_frame)

        expected_data_frame = self.SPARK.createDataFrame(
            [
                (
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ),
                (
                    [
                        AnalyzerResult("test@sample.com", "EMAIL", 6, 21),
                        AnalyzerResult("+65 62345678", "PHONE_NUMBER", 32, 44),
                    ],
                    [
                        AnalyzerResult("+65 62345678", "PHONE_NUMBER", 10, 22),
                        AnalyzerResult("+65 62345678", "PHONE_NUMBER", 33, 45),
                    ],
                ),
            ],
            self.schema,
        )

        self.assertEqual(actual.schema, expected_data_frame.schema)
        self.assertEqual(actual.collect(), expected_data_frame.collect())

    def test_analyze_data_frame_returns_empty_data_frame_when_there_are_no_PII_values(
        self,
    ):
        test_data_frame = self.SPARK.createDataFrame(
            [("No", "Personal"), ("Data", "Inside")], ["summary", "phone number"]
        )

        actual = self.pii_detector.get_analyzer_results(test_data_frame)

        expected_data_frame = self.SPARK.createDataFrame(
            [([], []), ([], [])], self.schema
        )

        self.assertEqual(actual.schema, expected_data_frame.schema)
        self.assertEqual(actual.collect(), expected_data_frame.collect())

    def test_get_pii_list_returns_list_of_pii_words_given_row_of_list_of_analyzer_results(
        self,
    ):
        test_row = Row(
            summary=[
                AnalyzerResult("S0000001I", "NRIC", 38, 47),
                AnalyzerResult("S0000002I", "NRIC", 38, 47),
            ],
            phone_number=[AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
        )
        actual = self.pii_detector._get_pii_list(test_row)
        expected = ["S0000001I", "S0000002I", "+65 62345678"]
        self.assertEqual(actual, expected)

    def test_get_pii_list_returns_empty_lists_no_analyzer_results(self):
        test_row = Row(summary=[], phone_number=[])
        actual = self.pii_detector._get_pii_list(test_row)
        expected = []
        self.assertEqual(actual, expected)

    def test_get_redacted_text_returns_redacted_data_frame(self):
        test_report_data_frame = self.SPARK.createDataFrame(
            [
                (
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ),
                (
                    [
                        AnalyzerResult("test@sample.com", "EMAIL", 6, 21),
                        AnalyzerResult("+65 62345678", "PHONE_NUMBER", 32, 44),
                    ],
                    [
                        AnalyzerResult("+65 62345678", "PHONE_NUMBER", 10, 22),
                        AnalyzerResult("+65 62345678", "PHONE_NUMBER", 33, 45),
                    ],
                ),
            ],
            self.schema,
        )

        test_input_data_frame = self.SPARK.createDataFrame(
            [
                (
                    "First President of Singapore NRIC was S0000001I",
                    "Some examples of phone numbers are +65 62345678",
                ),
                (
                    "email test@sample.com and phone +65 62345678",
                    "Phone one +65 62345678 Phone two +65 62345678",
                ),
            ],
            ["summary", "phone number"],
        )

        actual = self.pii_detector.get_redacted_text(
            test_input_data_frame, test_report_data_frame
        )

        expected = self.SPARK.createDataFrame(
            [
                (
                    "First President of Singapore NRIC was ",
                    "Some examples of phone numbers are ",
                ),
                ("email  and phone ", "Phone one  Phone two "),
            ],
            ["summary", "phone number"],
        )

        self.assertEqual(actual.schema, expected.schema)
        self.assertEqual(actual.collect(), expected.collect())

    def test_get_redacted_text_returns_same_data_frame_if_analyzer_results_are_empty(
        self,
    ):
        test_report_data_frame = self.SPARK.createDataFrame(
            [([], []), ([], [])], self.schema
        )

        test_input_data_frame = self.SPARK.createDataFrame(
            [("No", "Personal"), ("Data", "Inside")], ["summary", "phone number"]
        )

        actual = self.pii_detector.get_redacted_text(
            test_input_data_frame, test_report_data_frame
        )

        expected = self.SPARK.createDataFrame(
            [("No", "Personal"), ("Data", "Inside")], ["summary", "phone number"]
        )

        self.assertEqual(actual.schema, expected.schema)
        self.assertEqual(actual.collect(), expected.collect())



from pyspark.sql import SparkSession
from pii_anonymizer.spark.constants import FILE_PATH
import pyspark.sql.functions as f


class CsvParser:
    def __init__(self, spark: SparkSession, config):
        self.__validate_config(config)
        self.input_path = config["file_path"]
        self.delimiter = (
            config["delimiter"]
            if "delimiter" in config and config["delimiter"]
            else ","
        )
        self.spark = spark

    def __validate_config(self, config):
        if FILE_PATH not in config or not config[FILE_PATH]:
            raise ValueError("Config 'file_path' needs to be provided for parsing")

    def parse(self):
        df = self.spark.read.load(
            self.input_path,
            format="csv",
            sep=self.delimiter,
            header="true",
            inferSchema="true",
        )

        return df



from pii_anonymizer.spark.analyze.detectors.base_detector import BaseDetector
from pii_anonymizer.spark.analyze.utils.regex import RegEx


class EmailDetector(BaseDetector):
    def __init__(self):
        self.name = "EMAIL"
        self.pattern = (
            RegEx()
            .one_of("a-zA-Z0-9_.+-")
            .one_or_more_occurrences()
            .literal("@")
            .one_of("a-zA-Z0-9-")
            .one_or_more_occurrences()
            .literal("\\.")
            .one_of("a-zA-Z0-9-.")
            .one_or_more_occurrences()
            .build()
        )

    def get_name(self):
        return self.name

    def get_pattern(self):
        return self.pattern



import argparse
import os

default_file_path = "pii-anonymizer.json"

def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--config-file", help="config file to run the tool")
    args = parser.parse_args()
    
    if args.config_file:
        return args.config_file
    
    if os.path.exists(default_file_path):
        return default_file_path
    
    raise ValueError(f"Config file path or default file required. Please provide config file path or create {default_file_path}")


import importlib
import pkgutil
import inspect
import sys

import pandas as pd

import pii_anonymizer.standalone.analyze.detectors
from pii_anonymizer.standalone.analyze.detectors.base_detector import BaseDetector
from pii_anonymizer.standalone.anonymize.drop_anonymizer import DropAnonymizer
from pii_anonymizer.standalone.anonymize.anonymizer_result import AnonymizerResult


# TODO : refactor this to use the annotations instead of the module path.
class PIIDetector:
    def __init__(self):
        self.detectors = self.__get_detector_instances()

    def __get_detector_modules(self):
        modules = [
            modname
            for importer, modname, ispkg in pkgutil.walk_packages(
                path=pii_anonymizer.standalone.analyze.detectors.__path__,
                prefix=pii_anonymizer.standalone.analyze.detectors.__name__ + ".",
            )
            if "tests" not in modname
        ]
        return modules

    def __get_detector_instances(self):
        modules = self.__get_detector_modules()
        detectors = []
        for module in modules:
            importlib.import_module(module)
            classes = inspect.getmembers(sys.modules[module], inspect.isclass)
            for class_name, class_type in classes:
                if class_name != "BaseDetector" and issubclass(
                    class_type, BaseDetector
                ):
                    detectors.append(class_type())
        return detectors

    # TODO : Should we make this static?
    def analyze_and_redact(self, text: str):
        analyzer_results = []
        for detector in self.detectors:
            analyzer_results = analyzer_results + detector.execute(text)
        redacted_text = DropAnonymizer.redact(text, analyzer_results)
        return AnonymizerResult(redacted_text, analyzer_results)

    def __contains_pii(self, results):
        for result in results:
            if len(result.analyzer_results) > 0:
                return True
        return False

    def analyze_data_frame(self, input_data_frame):
        result_df = input_data_frame.applymap(self.analyze_and_redact)
        return result_df.applymap(lambda x: x.analyzer_results), result_df.applymap(
            lambda x: x.redacted_text
        )



from unittest import TestCase
from unittest.mock import patch, MagicMock

import pandas as pd
from pandas._testing import assert_frame_equal
from freezegun import freeze_time
from pyspark.sql.session import SparkSession
from pyspark.sql.types import (
    Row,
    StructField,
    StructType,
    ArrayType,
    StringType,
    LongType,
)
from pii_anonymizer.spark.report.report_generator import ReportGenerator
from pii_anonymizer.spark.analyze.utils.analyzer_result import AnalyzerResult


class TestReportGenerator(TestCase):
    @patch(
        "pii_anonymizer.spark.report.report_generator.ReportGenerator.setup_logging_config"
    )
    def setUp(self, mock_setup_logging_config):
        self.SPARK = (
            SparkSession.builder.master("local")
            .appName("Test PIIDetector")
            .getOrCreate()
        )

        self.array_structtype = StructType(
            [
                StructField("end", LongType(), False),
                StructField("start", LongType(), False),
                StructField("text", StringType(), False),
                StructField("type", StringType(), False),
            ]
        )
        self.schema = StructType(
            [
                StructField(
                    "summary", ArrayType(self.array_structtype, True), nullable=False
                ),
                StructField(
                    "phone number",
                    ArrayType(self.array_structtype, True),
                    nullable=False,
                ),
            ]
        )
        self.report_generator_high_level = ReportGenerator(
            config={"location": "abc", "level": "high"}
        )
        mock_setup_logging_config.assert_called_with()
        self.report_generator_medium_level = ReportGenerator(
            config={"location": "abc", "level": "medium"}
        )
        mock_setup_logging_config.assert_called_with()

    def test_high_level_reporting_returns_columns_with_PII_values_when_given_a_results_data_frame(
        self,
    ):
        test_data_frame = self.SPARK.createDataFrame(
            [
                (
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ),
                (
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ),
            ],
            self.schema,
        )
        expected_data_frame = pd.DataFrame(
            {"Columns with PII values": ["summary", "phone number"]}
        )
        self.assertCountEqual(
            expected_data_frame,
            self.report_generator_high_level.generate_report_content(test_data_frame),
        )

    def test_medium_level_reporting_returns_data_frame_with_detectors_and_column_details(
        self,
    ):
        test_data_frame = self.SPARK.createDataFrame(
            [
                (
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ),
                (
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ),
            ],
            self.schema,
        )

        expected_data_frame = pd.DataFrame(
            {
                "summary": [(1, "50.0%"), 0, (1, "50.0%")],
                "phone number": [0, (1, "50.0%"), (1, "50.0%")],
            },
            index=["NRIC", "EMAIL", "PHONE_NUMBER"],
        )

        self.assertCountEqual(
            list(expected_data_frame),
            self.report_generator_medium_level.spark_generate_medium_level_report(
                test_data_frame
            ),
        )

    def test_that_medium_level_reporting_returns_correct_data_frame(self):
        test_data_frame = self.SPARK.createDataFrame(
            [
                (
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ),
                (
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ),
            ],
            self.schema,
        )

        expected_data_frame = pd.DataFrame(
            {
                "summary": [(1, "50.0%"), 0, (1, "50.0%")],
                "phone number": [0, (1, "50.0%"), (1, "50.0%")],
            },
            index=["NRIC", "EMAIL", "PHONE_NUMBER"],
        )

        actual = self.report_generator_medium_level.spark_generate_medium_level_report(
            test_data_frame
        )
        assert_frame_equal(actual, expected_data_frame)

    @patch("logging.info")
    @patch(
        "pii_anonymizer.spark.report.report_generator.ReportGenerator.generate_report_content"
    )
    def test_generate_report_calls_content_generate_report_content_and_logs_it(
        self, mock_generate_content, mock_logging
    ):
        test_data_frame = self.SPARK.createDataFrame(
            [
                (
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ),
                (
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ),
            ],
            self.schema,
        )
        mock_generate_content.return_value = pd.DataFrame(
            {"Columns with PII values": ["summary", "phone number"]}
        )
        mock_logging.return_value = None
        expected_result = self.report_generator_high_level.generate(test_data_frame)
        self.assertCountEqual(expected_result, mock_generate_content.return_value)

    @freeze_time("2019-05-29 01:01:03")
    @patch("logging.FileHandler")
    @patch("logging.Logger.addHandler")
    @patch("genericpath.exists")
    def test_creation_of_the_report_file_if_not_present(
        self, mock_file_exists, mock_add_handler, mock_file_handler
    ):
        mock_file_exists.return_value = False
        mock_file_handler.return_value = MagicMock()
        self.report_generator_high_level.setup_logging_config()
        mock_file_handler.assert_called_with(
            filename="abc/report_20190529.log", mode="x"
        )
        mock_add_handler.assert_called_with(mock_file_handler.return_value)

    @freeze_time("2019-05-29 01:01:03")
    @patch("logging.FileHandler")
    @patch("logging.Logger.addHandler")
    @patch("os.path.exists")
    def test_appending_to_report_file_if_already_present(
        self, mock_file_exists, mock_add_handler, mock_file_handler
    ):
        mock_file_exists.return_value = True
        mock_file_handler.return_value = MagicMock()
        self.report_generator_high_level.setup_logging_config()
        mock_file_handler.assert_called_with(
            filename="abc/report_20190529.log", mode="a"
        )
        mock_add_handler.assert_called_with(mock_file_handler.return_value)

    def test_that_when_report_dataframe_contains_only_empty_lists_it_is_considered_empty(
        self,
    ):
        test_data_frame = self.SPARK.createDataFrame([([], []), ([], [])], self.schema)

        actual = self.report_generator_medium_level.is_empty_report_dataframe(
            test_data_frame
        )
        expected = True

        self.assertEqual(actual, expected)

    def test_that_when_report_dataframe_contains_some_text_it_is_not_considered_empty(
        self,
    ):
        test_data_frame = self.SPARK.createDataFrame(
            [([AnalyzerResult("S0000001I", "NRIC", 38, 47)], []), ([], [])], self.schema
        )

        actual = self.report_generator_medium_level.is_empty_report_dataframe(
            test_data_frame
        )
        expected = False

        self.assertEqual(actual, expected)

    def test_that_get_detector_results_returns_list_of_detector_results(self):
        columns = ["summary", "phone_number"]
        test_row = Row(
            summary=[Row(end=47, start=38, text="S0000001I", type="NRIC")],
            phone_number=[Row(end=60, start=45, text="test@sample.com", type="EMAIL")],
        )
        actual = self.report_generator_medium_level._get_detector_results(
            test_row, columns
        )
        expected = [(("summary", "NRIC"), 1), (("phone_number", "EMAIL"), 1)]
        self.assertEqual(actual, expected)

    def test_that_get_detector_results_returns_list_of_detector_results_if_column_is_empty(
        self,
    ):
        columns = ["summary", "phone_number"]
        test_row = Row(
            summary=[Row(end=47, start=38, text="S0000001I", type="NRIC")],
            phone_number=[],
        )
        actual = self.report_generator_medium_level._get_detector_results(
            test_row, columns
        )
        expected = [(("summary", "NRIC"), 1), (("phone_number", "no_pii"), 1)]
        self.assertEqual(actual, expected)



from unittest import TestCase

from pii_anonymizer.standalone.analyze.utils.regex import RegEx


class TestRegEx(TestCase):

    # Testing one_of
    def test_when_one_of_param_is_empty_throws_error(self):
        self.assertRaises(ValueError, lambda: RegEx().one_of("").build())

    def test_when_valid_input_is_passed_one_of_returns_correct_output(self):
        self.assertEqual("[AB]", RegEx().one_of("AB").build())
        self.assertEqual("[357]", RegEx().one_of("357").build())

    # Testing num_occurrences
    def test_when_non_positive_number_of_occurrences_throws_error(self):
        self.assertRaises(ValueError, lambda: RegEx().num_occurrences(-7).build())
        self.assertRaises(ValueError, lambda: RegEx().num_occurrences(0).build())

    def test_when_valid_input_is_passed_num_occurrences_returns_correct_output(self):
        self.assertEqual("{7}", RegEx().num_occurrences(7).build())

    # Testing any_digit
    def test_when_any_digit_returns_correct_output(self):
        self.assertEqual("\\d", RegEx().any_digit().build())

    def __assert_value_error_is_raised(self, fn, msg):
        with self.assertRaises(ValueError) as ve:
            fn()
        self.assertEqual(str(ve.exception), msg)

    def __assert_type_error_is_raised(self, fn, msg):
        with self.assertRaises(TypeError) as ve:
            fn()
        self.assertEqual(str(ve.exception), msg)

    # Testing range
    def test_when_range_is_incomplete(self):
        single_character = "Range boundaries should be single character"
        self.__assert_value_error_is_raised(
            lambda: RegEx().range("", "Z").build(), single_character
        )
        self.__assert_value_error_is_raised(
            lambda: RegEx().range("0", "").build(), single_character
        )
        self.__assert_value_error_is_raised(
            lambda: RegEx().range("01", "9").build(), single_character
        )
        self.__assert_value_error_is_raised(
            lambda: RegEx().range("A", "YZ").build(), single_character
        )

    def test_when_invalid_range_boundaries_are_provided(self):
        less_than_end = "Range start should be less than end"
        self.__assert_value_error_is_raised(
            lambda: RegEx().range("B", "A").build(), less_than_end
        )
        self.__assert_value_error_is_raised(
            lambda: RegEx().range("9", "0").build(), less_than_end
        )

    def test_when_valid_input_is_passed_range_returns_correct_output(self):
        self.assertEqual("[A-Z]", RegEx().range("A", "Z").build())
        self.assertEqual("[0-9]", RegEx().range("0", "9").build())

    # Testing range_occurrences
    def test_when_invalid_numeric_range_boundaries_are_provided(self):
        less_than_end = "Range start should be less than end"
        self.__assert_value_error_is_raised(
            lambda: RegEx().range_occurrences(9, 0).build(), less_than_end
        )

    def test_when_invalid_input_for_range_occurrences_throws_error(self):
        range_should_be_integers = "Range should be integers"
        self.__assert_type_error_is_raised(
            lambda: RegEx().range_occurrences(1.2, 2).build(), range_should_be_integers
        )
        self.__assert_type_error_is_raised(
            lambda: RegEx().range_occurrences("A", 9).build(), range_should_be_integers
        )

    def test_when_valid_input_is_passed_range_occurrences_returns_correct_output(self):
        self.assertEqual("{0,9}", RegEx().range_occurrences(0, 9).build())

    # Testing one_or_more_occurrences
    def test_when_valid_input_is_passed_one_or_more_occurrences_returns_correct_output(
        self,
    ):
        self.assertEqual("+", RegEx().one_or_more_occurrences().build())

    # Testing zero_or_more_occurrences
    def test_when_valid_input_is_passed_zero_or_more_occurrences_returns_correct_output(
        self,
    ):
        self.assertEqual("*", RegEx().zero_or_more_occurrences().build())

    # Testing zero_or_one_occurrences
    def test_when_valid_input_is_passed_zero_or_one_occurrences_returns_correct_output(
        self,
    ):
        self.assertEqual("?", RegEx().zero_or_one_occurrences().build())

    # Testing literal
    def test_when_valid_input_is_passed_literal_returns_correct_output(self):
        self.assertEqual("@", RegEx().literal("@").build())

    # Testing boundary
    def test_boundary(self):
        self.assertEqual("\\b", RegEx().boundary().build())

    # Testing complex inputs
    def test_builds_correct_pattern_for_NRIC(self):
        self.assertEqual(
            "[AIR]\\d{7}[A-Z]",
            RegEx()
            .one_of("AIR")
            .any_digit()
            .num_occurrences(7)
            .range("A", "Z")
            .build(),
        )

        self.assertEqual(
            "[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\\.[a-zA-Z0-9-.]+",
            RegEx()
            .one_of("a-zA-Z0-9_.+-")
            .one_or_more_occurrences()
            .literal("@")
            .one_of("a-zA-Z0-9-")
            .one_or_more_occurrences()
            .literal("\\.")
            .one_of("a-zA-Z0-9-.")
            .one_or_more_occurrences()
            .build(),
        )



import os
import sys

sys.path.append(os.path.abspath("."))
import json

from pii_anonymizer.standalone.report.report_generator import ReportGenerator
from pii_anonymizer.standalone.acquire.csv_parser import CsvParser
from pii_anonymizer.standalone.analyze.detectors.pii_detector import PIIDetector
from pii_anonymizer.standalone.constants import ACQUIRE, REPORT
from pii_anonymizer.standalone.write.csv_writer import CsvWriter


class DPFMain:
    def __init__(self, config_file_path):
        with open(config_file_path) as config_file:
            self.config = json.load(config_file)

    # TODO : validate the config for the stages right here
    def run(self):
        parsed_data_frame = CsvParser(config=self.config[ACQUIRE]).parse()
        pii_analysis_report, redacted_data_frame = PIIDetector().analyze_data_frame(
            parsed_data_frame
        )
        if pii_analysis_report.empty:
            print("NO PII VALUES WERE FOUND!")
        else:
            ReportGenerator(config=self.config[REPORT]).generate(
                results_df=pii_analysis_report,
            )
        CsvWriter(config=self.config).write_csv(df=redacted_data_frame)


# output_directory needs to be obtained from the config json file as a parameter in the 'anonymize' section.


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pii_anonymizer',
 'pii_anonymizer.common',
 'pii_anonymizer.spark',
 'pii_anonymizer.spark.acquire',
 'pii_anonymizer.spark.acquire.tests',
 'pii_anonymizer.spark.analyze',
 'pii_anonymizer.spark.analyze.detectors',
 'pii_anonymizer.spark.analyze.detectors.tests',
 'pii_anonymizer.spark.analyze.utils',
 'pii_anonymizer.spark.report',
 'pii_anonymizer.spark.report.tests',
 'pii_anonymizer.spark.write',
 'pii_anonymizer.spark.write.tests',
 'pii_anonymizer.standalone',
 'pii_anonymizer.standalone.acquire',
 'pii_anonymizer.standalone.acquire.tests',
 'pii_anonymizer.standalone.analyze',
 'pii_anonymizer.standalone.analyze.detectors',
 'pii_anonymizer.standalone.analyze.detectors.tests',
 'pii_anonymizer.standalone.analyze.utils',
 'pii_anonymizer.standalone.analyze.utils.tests',
 'pii_anonymizer.standalone.anonymize',
 'pii_anonymizer.standalone.anonymize.tests',
 'pii_anonymizer.standalone.report',
 'pii_anonymizer.standalone.report.tests',
 'pii_anonymizer.standalone.tests',
 'pii_anonymizer.standalone.tests.config',
 'pii_anonymizer.standalone.write',
 'pii_anonymizer.standalone.write.tests']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.5.0,<2.0.0', 'pyspark>=3.3.0,<3.4.0']

setup_kwargs = {
    'name': 'pii-anonymizer',
    'version': '0.1.0',
    'description': 'Data Protection Framework is a python library/command line application for identification, anonymization and de-anonymization of Personally Identifiable Information data.',
    'long_description': '# Data Protection Framework\nData Protection Framework is a python library/command line application for identification, anonymization and de-anonymization of Personally Identifiable Information data.\n\nThe framework aims to work on a two-fold principle for detecting PII:\n1. Using RegularExpressions using a pattern\n2. Using NLP for detecting NER (Named Entity Recognitions)\n\n## Features and Current Status\n\n### Completed\n * Following Global detectors have been completed:\n   * [x] EMAIL_ADDRESS :  An email address identifies the mailbox that emails are sent to or from. The maximum length of the domain name is 255 characters, and the maximum length of the local-part is 64 characters.\n   * [x] CREDIT_CARD_NUMBER : A credit card number is 12 to 19 digits long. They are used for payment transactions globally.\n\n * Following detectors specific to Singapore have been completed:\n   * [x] PHONE_NUMBER : A telephone number.\n   * [x] FIN/NRIC : A unique set of nine alpha-numeric characters on the Singapore National Registration Identity Card.\n\n * Following anonymizers have been added\n    * [x] Redaction: Deletes all or part of a detected sensitive value.\n    * [x] Encryption :  Encrypts the original sensitive data value using a cryptographic key. Cloud DLP supports several types of tokenization, including transformations that can be reversed, or "re-identified."\n\n### TO-DO\nFollowing features  are part of the backlog with more features coming soon\n * Detectors:\n    * [ ] NAME\n    * [ ] ADDRESS\n * Anonymizers:\n    * [ ] Masking: Replaces a number of characters of a sensitive value with a specified surrogate character, such as a hash (#) or asterisk (*).\n    * [ ] Bucketing: "Generalizes" a sensitive value by replacing it with a range of values. (For example, replacing a specific age with an age range,\n    or temperatures with ranges corresponding to "Hot," "Medium," and "Cold.")\n    * [ ] Replacement: Replaces a detected sensitive value with a specified surrogate value.\n\n\nYou can have a detailed at upcoming features and backlog in this [Github Board](https://github.com/thoughtworks-datakind/anonymizer/projects/1?fullscreen=true)\n\n## Development setup\n\nClone the [repo](https://github.com/thoughtworks-datakind/anonymizer) and follow the below instructions:  <br/>\n_Assuming that $pwd is where you cloned the repo_\n2. Setup venv : `./bin/setup_venv_locally.sh`\n3. Activate venv : `source ./.venv/bin/activate`\n4. Install dependencies : `pip install -r requirements-dev.txt`\n\n### Config JSON\nAn example for the config JSON is located at `<PROJECT_ROOT>/config.json`\n```\n{\n  "acquire": {\n    "file_path": <FILE PATH TO YOUR INPUT CSV>,\n    "delimiter": <YOUR CSV DELIMITER>\n  },\n  "analyze": {\n\n  },\n  "report" : {\n    "location" : <PATH TO YOUR REPORT OUTPUT FOLDER>,\n    "level" : <LOG LEVEL>\n  },\n  "anonymize": {\n    "output_file_path" : <PATH TO YOUR CSV OUTPUT FOLDER>\n  }\n}\n```\n\n### Running Tests\nUpdate this file first `<PROJECT_ROOT>/src/tests/config/test_config.json` \\\nYou can run the tests by triggering shell script located at `<PROJECT_ROOT>/bin/run_tests.sh`\n\n### Trying out on local\n\n##### Anonymizing a delimited csv file\n1. Set up a JSON config file similar to the one seen at the project root.\nIn the \'acquire\' section of the json, populate the input file path and the delimiter.\nIn the \'report\' section, provide the output path, where you want the PII detection report to be generated.\nA \'high\' level report just calls out which columns have PII attributes.\nA \'medium\' level report calls out the percentage of PII in each column and the associated PII (email, credit card, etc)type for the same.\n2. Run the main class - `python src/dpf_main.py --config <absolute path of the config file>`\nYou should see the report being appended to the file named \'report_\\<date\\>.log\' in the output path specified in the\nconfig file.\n\n### Packaging\nRun `python setup.py bdist_wheel` and the `.whl` file will be created in the `dist` folder.\n\n### Spark-submit\nTo run spark-submit locally, you can run the following command\n`spark-submit --py-files dist/SomePackage-*.whl src_spark/main.py --config config.json`\n\n\n### Licensing\nDistributed under the MIT license. See ``LICENSE`` for more information.\n\n\n### Contributing\n\nYou want to help out? _Awesome_!\n',
    'author': 'Thoughtworks',
    'author_email': 'thoughtworks@thoughtworks.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from unittest import TestCase
from unittest.mock import patch, MagicMock

import os
import pandas as pd
from freezegun import freeze_time

from pii_anonymizer.standalone.report.report_generator import ReportGenerator
from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult


class TestReportGenerator(TestCase):
    @patch(
        "pii_anonymizer.standalone.report.report_generator.ReportGenerator.setup_logging_config"
    )
    def setUp(self, mock_setup_logging_config):
        self.report_generator_high_level = ReportGenerator(
            config={"location": "abc", "level": "high"}
        )
        mock_setup_logging_config.assert_called_with()
        self.report_generator_medium_level = ReportGenerator(
            config={"location": "abc", "level": "medium"}
        )
        mock_setup_logging_config.assert_called_with()

    def test_high_level_reporting_returns_columns_with_PII_values_when_given_a_results_data_frame(
        self,
    ):
        result_data_frame = pd.DataFrame(
            {
                "summary": [
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ],
                "phone number": [
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ],
            }
        )
        expected_data_frame = pd.DataFrame(
            {"Columns with PII values": ["summary", "phone number"]}
        )
        self.assertCountEqual(
            expected_data_frame,
            self.report_generator_high_level.generate_report_content(result_data_frame),
        )

    def test_medium_level_reporting_returns_data_frame_with_detectors_and_column_details(
        self,
    ):
        result_data_frame = pd.DataFrame(
            {
                "summary": [
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ],
                "phone number": [
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ],
            }
        )
        expected_data_frame = pd.DataFrame(
            {
                "summary": pd.Series({"NRIC": (1, "50%"), "EMAIL": (1, "50%")}),
                "phone number": pd.Series({"PHONE_NUMBER": (2, "100%")}),
            }
        )
        self.assertCountEqual(
            list(expected_data_frame),
            self.report_generator_medium_level.generate_report_content(
                result_data_frame
            ),
        )

    def test_calculate_detector_stats_returns_detector_counts_and_percentages(self):
        result_column_values = pd.Series(
            [
                [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
            ]
        )
        actual_result = (
            self.report_generator_medium_level.calculate_detector_stats_for_each_column(
                result_column_values
            )
        )
        expected_result = {"NRIC": (1, "33.33%"), "EMAIL": (2, "66.67%")}
        self.assertCountEqual(expected_result, actual_result)

    @patch("logging.info")
    @patch(
        "pii_anonymizer.standalone.report.report_generator.ReportGenerator.generate_report_content"
    )
    def test_generate_report_calls_content_generate_report_content_and_logs_it(
        self, mock_generate_content, mock_logging
    ):
        result_data_frame = pd.DataFrame(
            {
                "summary": [
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ],
                "phone number": [
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ],
            }
        )
        mock_generate_content.return_value = pd.DataFrame(
            {"Columns with PII values": ["summary", "phone number"]}
        )
        mock_logging.return_value = None
        expected_result = self.report_generator_high_level.generate(result_data_frame)
        self.assertCountEqual(expected_result, mock_generate_content.return_value)

    @freeze_time("2019-05-29 01:01:03")
    @patch("logging.FileHandler")
    @patch("logging.Logger.addHandler")
    @patch("genericpath.exists")
    def test_creation_of_the_report_file_if_not_present(
        self, mock_file_exists, mock_add_handler, mock_file_handler
    ):
        mock_file_exists.return_value = False
        mock_file_handler.return_value = MagicMock()
        self.report_generator_high_level.setup_logging_config()
        mock_file_handler.assert_called_with(
            filename="abc/report_20190529.log", mode="x"
        )
        mock_add_handler.assert_called_with(mock_file_handler.return_value)

    @freeze_time("2019-05-29 01:01:03")
    @patch("logging.FileHandler")
    @patch("logging.Logger.addHandler")
    @patch("os.path.exists")
    def test_appending_to_report_file_if_already_present(
        self, mock_file_exists, mock_add_handler, mock_file_handler
    ):
        mock_file_exists.return_value = True
        mock_file_handler.return_value = MagicMock()
        self.report_generator_high_level.setup_logging_config()
        mock_file_handler.assert_called_with(
            filename="abc/report_20190529.log", mode="a"
        )
        mock_add_handler.assert_called_with(mock_file_handler.return_value)



from pii_anonymizer.standalone.analyze.detectors.base_detector import BaseDetector
from pii_anonymizer.standalone.analyze.utils.regex import RegEx


class PhoneNumberDetector(BaseDetector):
    def __init__(self):
        self.name = "PHONE_NUMBER"
        regex_pipe = RegEx().pipe().build()

        regex_with_country_code_and_no_space = "(\\+65?\\s?[689]\\d{7})"
        regex_with_country_code_and_single_space = "(\\+65?\\s?[689]\\d{3} \\d{4})"
        regex_no_country_code_and_no_space = "([689]\\d{7})"
        regex_no_country_code_and_single_space = "([689]\\d{3} \\d{4})"
        regex_with_country_code_in_brackets_and_no_space = "([(]65[)]\\s?[689]\\d{7})"
        regex_with_country_code_in_brackets_and_single_space = (
            "([(]65[)]\\s?[689]\\d{3} \\d{4})"
        )

        self.pattern = (
            regex_with_country_code_and_no_space
            + regex_pipe
            + regex_with_country_code_and_single_space
            + regex_pipe
            + regex_no_country_code_and_no_space
            + regex_pipe
            + regex_no_country_code_and_single_space
            + regex_pipe
            + regex_with_country_code_in_brackets_and_no_space
            + regex_pipe
            + regex_with_country_code_in_brackets_and_single_space
        )

    def get_name(self):
        return self.name

    def get_pattern(self):
        return self.pattern



from unittest import TestCase
from pii_anonymizer.standalone.anonymize.drop_anonymizer import DropAnonymizer
from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult


class TestDropAnonymizer(TestCase):
    def test_redact_for_single_analyzer_result(self):
        text = "text containing pii"
        analyzer_results = [AnalyzerResult("pii", "PII_DETECTOR", 16, 18)]
        result = DropAnonymizer.redact(text, analyzer_results)
        self.assertEqual(result, "text containing ")

    def test_redact_for_multiple_analyzer_results(self):
        text = "text containing pii1 and pii2"
        analyzer_results = [
            AnalyzerResult("pii1", "PII_DETECTOR", 16, 19),
            AnalyzerResult("pii2", "PII_DETECTOR", 25, 28),
        ]
        result = DropAnonymizer.redact(text, analyzer_results)
        self.assertEqual(result, "text containing  and ")



import pandas as pd
from unittest import TestCase

from pii_anonymizer.standalone.analyze.detectors.pii_detector import PIIDetector
from pii_anonymizer.standalone.analyze.utils.analyzer_result import AnalyzerResult
from pii_anonymizer.standalone.anonymize.anonymizer_result import AnonymizerResult


class TestPIIDetector(TestCase):
    def setUp(self):
        self.pii_detector = PIIDetector()

    def test_should_detect_and_redact_nric_in_text(self):
        actual = self.pii_detector.analyze_and_redact(
            "First President of Singapore NRIC was S0000001I"
        )
        expected = AnonymizerResult(
            "First President of Singapore NRIC was ",
            [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
        )
        self.assertEqual(actual, expected)

    def test_should_detect_and_redact_email_in_text(self):
        actual = self.pii_detector.analyze_and_redact(
            "A typical email id would look something like test@sample.com"
        )
        expected = AnonymizerResult(
            "A typical email id would look something like ",
            [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
        )
        self.assertEqual(actual, expected)

    def test_should_detect_and_redact_phone_in_text(self):
        actual = self.pii_detector.analyze_and_redact(
            "Some examples of phone numbers are +65 62345678"
        )
        expected = AnonymizerResult(
            "Some examples of phone numbers are ",
            [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
        )
        self.assertEqual(actual, expected)

    def test_should_detect_and_redact_all_pii_fields_in_text(self):
        actual = self.pii_detector.analyze_and_redact(
            """First President of Singapore NRIC was S0000001I.
                                         A typical email id would look something like test@sample.com"""
        )
        expected_redacted_text = """First President of Singapore NRIC was .
                                         A typical email id would look something like """

        expected = AnonymizerResult(
            expected_redacted_text,
            [
                AnalyzerResult("test@sample.com", "EMAIL", 135, 150),
                AnalyzerResult("S0000001I", "NRIC", 38, 47),
            ],
        )
        self.assertEqual(actual, expected)

    def test_analyze_returns_returns_same_text_and_no_results_when_no_PII_fields(self):
        input_text = """First President of Singapore NRIC was ABC.
                                         A typical email id would look something like test"""
        actual = self.pii_detector.analyze_and_redact(input_text)
        expected = AnonymizerResult(input_text, [])
        self.assertEqual(actual, expected)

    def test_analyze_data_frame_runs_analyze_against_each_cell_with_a_PII_value(self):
        test_data_frame = pd.DataFrame(
            {
                "summary": [
                    "First President of Singapore NRIC was S0000001I",
                    "A typical email id would look something like test@sample.com",
                ],
                "phone number": [
                    "Some examples of phone numbers are +65 62345678",
                    "Some examples of phone numbers are +65 62345678",
                ],
            }
        )

        actual, _ = self.pii_detector.analyze_data_frame(test_data_frame)

        expected_data_frame = pd.DataFrame(
            {
                "summary": [
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ],
                "phone number": [
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                    [AnalyzerResult("+65 62345678", "PHONE_NUMBER", 35, 47)],
                ],
            }
        )

        pd.testing.assert_frame_equal(expected_data_frame, actual)

    def test_analyze_data_frame_runs_analyze_against_each_cell_when_there_are_no_PII_values_returns_empty_data_frame(
        self,
    ):
        test_data_frame = pd.DataFrame(
            {
                "summary": [
                    "First President of Singapore NRIC was abcde",
                    "A typical email id would look something like test@t",
                ],
                "phone number": [
                    "Some examples of phone numbers are +34342",
                    "Some examples of phone numbers are +8909",
                ],
            }
        )
        expected_report = pd.DataFrame({"summary": [[], []], "phone number": [[], []]})
        expected_result = pd.DataFrame(
            {
                "summary": [
                    "First President of Singapore NRIC was abcde",
                    "A typical email id would look something like test@t",
                ],
                "phone number": [
                    "Some examples of phone numbers are +34342",
                    "Some examples of phone numbers are +8909",
                ],
            }
        )
        actual_report, actual_result = self.pii_detector.analyze_data_frame(
            test_data_frame
        )

        pd.testing.assert_frame_equal(expected_report, actual_report)
        pd.testing.assert_frame_equal(expected_result, actual_result)

    def test_analyze_data_frame_runs_analyze_only_on_cells_with_a_PII_value(self):
        test_data_frame = pd.DataFrame(
            {
                "summary": [
                    "First President of Singapore NRIC was S0000001I",
                    "A typical email id would look something like test@sample.com",
                ],
                "remarks": ["No sensitive data", "No sensitive data"],
            }
        )

        actual_report, actual_result = self.pii_detector.analyze_data_frame(
            test_data_frame
        )

        expected_report = pd.DataFrame(
            {
                "summary": [
                    [AnalyzerResult("S0000001I", "NRIC", 38, 47)],
                    [AnalyzerResult("test@sample.com", "EMAIL", 45, 60)],
                ],
                "remarks": [[], []],
            }
        )

        expected_result = pd.DataFrame(
            {
                "summary": [
                    "First President of Singapore NRIC was ",
                    "A typical email id would look something like ",
                ],
                "remarks": ["No sensitive data", "No sensitive data"],
            }
        )

        pd.testing.assert_frame_equal(expected_report, actual_report)
        pd.testing.assert_frame_equal(expected_result, actual_result)



from unittest import TestCase
from unittest.mock import patch

from pii_anonymizer.standalone.analyze.detectors.email_detector import EmailDetector


class TestEmailDetector(TestCase):
    def setUp(self):
        self.email_detector = EmailDetector()

    def test_get_name_returns_the_valid_detector_name(self):
        self.assertEqual(self.email_detector.get_name(), "EMAIL")

    def test_get_pattern_returns_compiled_regex(self):
        actual_value = self.email_detector.get_pattern()
        return_value = "[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\\.[a-zA-Z0-9-.]+"
        self.assertEqual(return_value, actual_value)

    def test_valid_email_gets_detected_correctly(self):
        self.assertEqual(len(self.email_detector.execute("abc@hotmail.com")), 1)

    def test_invalid_email_does_not_get_detected(self):
        self.assertEqual(len(self.email_detector.execute("@hotmail.com")), 0)
